#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "Ball.h"

namespace Ball{
int ball_detect(double *ret_disp, double *ret_degp){
	double dis=-999, deg=-999;
	IplImage *input_image;
	IplImage *output_image;

	img_update(INPUT_FILE, CAM_WIDTH, CAM_HEIGHT);
	input_image = cvLoadImage(INPUT_FILE, CV_LOAD_IMAGE_ANYCOLOR);
	output_image = cvCreateImage(cvGetSize(input_image), IPL_DEPTH_8U, 1);

	//coler threshold(2値化)->Erode & Dilate(収縮->膨張　白ノイズとり)->Canny edge(エッジ検出)->Smooth(平滑化)->HoughCircles(ハフ変換による円検出)
	col_threshold(input_image, output_image, THRE_H_MIN, THRE_H_MAX, THRE_S_MIN, THRE_S_MAX, THRE_V_MIN, THRE_V_MAX, CODE);
	cvErode(output_image, output_image, NULL, 2);
	cvDilate(output_image, output_image, NULL, 5);
	cvCanny(output_image, output_image, CANNY_THRESHOLD1, CANNY_THRESHOLD2, APERTURE_SIZE);
	cvSmooth(output_image, output_image, CV_GAUSSIAN, SMOOTH_PARAM1, SMOOTH_PARAM2, 0, 0);
	hough_circle(output_image, output_image, (double*)&dis, (double*)&deg);	//disとdegを参照することで、ボールまでの距離と角度が分かる
	*ret_disp = dis;
	*ret_degp = deg;

	cvReleaseImage(&input_image);
	cvReleaseImage(&output_image);
	return 0;
}

void col_threshold(IplImage *src, IplImage *dst,
			 int th1_min, int th1_max,
			 int th2_min, int th2_max,
			 int th3_min, int th3_max,
			 const char *code)
{
	int i, j;
	int nx, ny;
	unsigned char color[3];
	int mode;
	nx = src->width; ny = src->height;
	IplImage *colorImage = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 3);

	//RGBかHSVかで場合わけ
	if(!strcmp(code, "HSV")){
		mode = CV_BGR2HSV; //BGR->HSV
	}else{
		mode = CV_BGR2RGB; //BGR->RGB
	}

	cvCvtColor(src, colorImage, mode); //mode で指定した変換をsrcに施して，結果をcolorImageに代入

	for(i = 0; i < ny; i++){
		for(j = 0; j < nx; j++){
			//i行j列の点の色データを取得
			color[0] = (unsigned char)colorImage->imageData[3*nx*i + 3*j + 0]; //HorR
			color[1] = (unsigned char)colorImage->imageData[3*nx*i + 3*j + 1]; //SorG
			color[2] = (unsigned char)colorImage->imageData[3*nx*i + 3*j + 2]; //VorB
			//i行j列の点の閾値処理（th1~3に指定した範囲に入った色を255とする）
			if( th1_min <= color[0] && color[0] <= th1_max &&
				th2_min <= color[1] && color[1] <= th2_max && 
				th3_min <= color[2] && color[2] <= th3_max)
			{
				dst->imageData[nx*i + j] = 255;
			}else{
				dst->imageData[nx*i + j] = 0;
			}
		}
	}
	cvReleaseImage(&colorImage); //メモリ開放
}

void hough_circle(IplImage *src, IplImage *dst , double *dis, double *deg){
	int i;
	float *ball;
	CvMemStorage *storage = cvCreateMemStorage(0);
	CvSeq *circles;
	
	cvZero(dst); //出力画像の初期化
	circles = cvHoughCircles(src, storage, METHOD, DP, src->height/4, HOUGH_PARAM1, HOUGH_PARAM2, 0, 0); //円検出
	
	debug_printf("cvHoughCircle total: %d\n",circles->total); //検出した円の総数を表示
	if(circles->total == 1){
		for(i=0; i<100; i++){
			ball = (float*)cvGetSeqElem(circles, circles->total);
			cvCircle(dst, cvPoint(cvRound(ball[0]), cvRound(ball[1])), 3, CV_RGB(255, 255, 255), -1, 8, 0); //円の中心の描画
			cvCircle(dst, cvPoint(cvRound(ball[0]), cvRound(ball[1])), cvRound(ball[2]), CV_RGB(255, 255, 255), 3, 8, 0); //円の描画
			//円の中心のx,y座標=(ball[0],ball[1]),円の半径=ball[2]，

			*dis = BALL_2R * src->width/2 * SQRT3 / ball[2]; //距離の計算 視野角が60度であることを仮定すれば成り立つ．
			*deg = VISUAL_DEG * ( ball[0] - src->width/2 ) / src->width; //中心からのずれの計算 視野角が60度であることを仮定すれば成り立つ．
		}
	}	
	debug_printf("Distance = %lf [mm], Angle = %lf [deg]\n",*dis,*deg); //ボールまでの距離と，角度のずれ
}

void img_update(const char *str, int width, int height){
	char command[100];
	int sy_val;
	sprintf(command, "uvccapture -o%s -x%d -y%d -C32 -B160 -S50 \n", str, width, height);
	sy_val=system(command);
	if(sy_val == -1){
		fprintf(stderr,"system(%s) returned -1",command);
	}
	usleep(10*1000);
	return;
}
}
