#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <opencv/cxcore.h>
#include <opencv/cv.h>
#include <opencv/highgui.h>

#include "Number.h"

namespace Number{
void binarization(IplImage *src,IplImage *dst, int sflag){
	IplImage *gray = 0;
	IplImage *gaus = 0;

	gray = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U,1);
	gaus = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U,1);

	cvCvtColor(src, gray, CV_BGR2GRAY);	//グレースケール化
	cvSmooth(gray, gaus, CV_GAUSSIAN, 5, 0, 0, 0);	//平滑化(ガウシアンフィルタ)
	cvThreshold(gaus, dst, 90, 255, CV_THRESH_BINARY_INV | CV_THRESH_OTSU);	//２値化
	//cvThreshold(gray, dst, 90, 255, CV_THRESH_BINARY_INV | CV_THRESH_OTSU);	//２値化
	
#ifdef __DEBUG_IMG__
	if(sflag==1){
#if CV_VERSION_NUM == 1
		cvSaveImage("./img/gray.jpg", gray);
		cvSaveImage("./img/gaus.jpg", gaus);
#else
		cvSaveImage("./img/gray.jpg", gray, NULL);
		cvSaveImage("./img/gaus.jpg", gaus, NULL);
#endif
	}
#endif

	cvReleaseImage(&gray);
	cvReleaseImage(&gaus);
}

void get_img(const char *str,int width,int height){
	int cap;
	char command[100];
	//sprintf(command,"uvccapture -o%s -x%d -y%d\n",str,width,height);
	sprintf(command,"uvccapture -o%s -x%d -y%d\n",str,width,height);
	cap=system(command);
	usleep(10*1000);
	if(cap!=0){
		fprintf(stderr,"Capture fail!!\n");
	}
}

int labelling(IplImage *src, int pxy[10][4]){

	int label_num=0;
	int cntr_num;
	int i, j;
	//int label_num = 0;
	int x1[100], y1[100], x2[100], y2[100], width[100], height[100];
	int xy_h[4];
	CvMemStorage *storage = cvCreateMemStorage(0);
	CvSeq *contour = NULL;
	CvRect rect[100];
	IplImage *src2;

	//src2= cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, src->nChannels);
	src2= cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);
	cvCopyImage(src, src2);

	label_num = 0;


	for(i=0;i<10;i++){
		for(j=0;j<4;j++){
			pxy[i][j] = -1;
		}
	}

	// エリアの抽出条件
	/* 320x240 pixel の画像を50cm の距離で撮影した場合
	   1以外の数字   width = 60 ,  height = 106 
	   1の場合       widht = 13 ,  height = 106 
	   widht_min = 10, widht_max = 80, height_min = 90, height_min = 120
	*/

	int width_min, width_max, height_min, height_max;
	float mag ; // 320x240pixel, 50cm を基準とした、拡大（縮小）率
	mag = 1;
	//mag = 2 * 50. /150. ; 640x480pixel, 150cm の場合

	width_min = 10*mag; width_max =80*mag; 
	height_min = 90*mag; height_max =120*mag; 
	debug_printf("width_min=%d, width_max=%d, height_min=%d, height_max=%d\n", width_min, width_max, height_min, height_max);

	if(src == NULL) return -1;
	cntr_num = cvFindContours(src, storage, &contour, sizeof(CvContour), CV_RETR_LIST, CV_CHAIN_APPROX_SIMPLE, cvPoint(0,0));//輪郭検出(戻り値は取得した輪郭の全個数)
	debug_printf("抽出した輪郭の個数=%d\n",cntr_num);
	for(i = 0; i < cntr_num; i++){
		x1[i] = 1;
		y1[i] = 1;
		x2[i] = 1;
		y2[i] = 1;
		height[i] = 0;
		width[i] = 0;
	}

	for(i = 0; i < cntr_num; i++){	//輪郭の傾いていない外接四角形領域の座標を取得
		rect[i] = cvBoundingRect(contour, 1);
		x1[i] = rect[i].x;
		y1[i] = rect[i].y;
		x2[i] = rect[i].x + rect[i].width;
		y2[i] = rect[i].y + rect[i].height;
		height[i] = rect[i].height;
		width[i] = rect[i].width;
		debug_printf("エリア番号=%d   エリアの幅=%d    エリアの高さ=%d\n",i,width[i],height[i]);
		cvRectangle(src2, cvPoint(x1[i], y1[i]), cvPoint(x2[i], y2[i]), CV_RGB(255, 255, 255), 1, 8, 0);

	// エリアの抽出条件
		if(width[i] > width_min && width[i] < width_max && height[i] > height_min && height[i] < height_max){  

			cvRectangle(src, cvPoint(x1[i], y1[i]), cvPoint(x2[i], y2[i]), CV_RGB(255, 255, 255), 1, 8, 0);
			xy_h[0] = x1[i], xy_h[1] = y1[i], xy_h[2] = x2[i], xy_h[3] = y2[i];
			for(j=0;j<4;j++){
				pxy[label_num][j] = xy_h[j];
			}
			label_num++;
			debug_printf("エリア番号=%d   ラベル番号=%d\n",i,label_num);
		}

		contour = contour->h_next;
	}

#if CV_VERSION_NUM == 1
	cvSaveImage("./img/label-all.jpg", src2);
	cvSaveImage("./img/label.jpg", src);
#else
	cvSaveImage("./img/label-all.jpg", src2, NULL);
	cvSaveImage("./img/label.jpg", src, NULL);
#endif

//メモリストレージの解放
//	cvReleaseMemStorage (&storage);
//	cvReleaseImage(&dst);

	return label_num;
}

int number_detect(void){
//*****変数定義*****
	static int number_l=-1, number_r=-1;
	int caperr = 0;
	IplImage *input = 0;
	IplImage *binr = 0;
	IplImage *trim = 0;
	IplImage *binr_t = 0;
	CvRect rect;
	CvSize cvsize;

	int count = 0;
	int num_1=0, num_2=0;
	int x0_1=0, x0_2=0;
	int number = - 1;

//*****画像取得*****
	//caperr = get_img("./img/input.jpg", IMG_X, IMG_Y);
	get_img("./img/input.jpg", IMG_X, IMG_Y);
	if(caperr != 0){	 //キャプチャが失敗したときの例外処理
		return -1;
	}
	input = cvLoadImage("./img/input.jpg", CV_LOAD_IMAGE_ANYCOLOR);

//	cvNamedWindow("input", CV_WINDOW_AUTOSIZE);
//	cvShowImage("input", input);
//	cvWaitKey(0);

//*****二値化*****
	binr = cvCreateImage(cvGetSize(input), IPL_DEPTH_8U,1);
	binarization(input, binr,1);
#if CV_VERSION_NUM == 1
	cvSaveImage("./img/binr.jpg", binr);
#else
	cvSaveImage("./img/binr.jpg", binr, NULL);
#endif

//	cvNamedWindow("binr", CV_WINDOW_AUTOSIZE);	//コアダンプが起きるので消せない
//	cvShowImage("binr", binr);	//コアダンプが起きるので消せない
//	cvWaitKey(0);

//*****ラベリング*****

	int nums;
	int xy[10][4];
	
	nums=labelling(binr, xy);
	if(nums < 0){
		fprintf(stderr,"labelling error\n");
		return -1;
	}

	debug_printf("トリミングエリアとして抽出されたエリアの数 = %d\n",nums);

//条件にあったラベリング領域について、２個の数字が認識されるまで、トリミング、二値化、
//数字認識処理を行う

	int k;
	for(k=0;k<nums;k++){

		debug_printf("triming number = %d\n",k);

	//*****トリミング*****
		rect = cvRect(xy[k][0], xy[k][1], xy[k][2] - xy[k][0], xy[k][3] - xy[k][1]);
		cvsize = cvSize(xy[k][2] - xy[k][0] + 2, xy[k][3] - xy[k][1] + 2);

		debug_printf("triming ...");
		triming(input, rect);
		debug_printf("OK. \n");

		trim = cvLoadImage("./img/trim.jpg", CV_LOAD_IMAGE_COLOR);


	//*****トリミング画像の二値化*****

		binr_t = cvCreateImage(cvGetSize(trim), IPL_DEPTH_8U,1);
		debug_printf("binarization ...");
		binarization(trim, binr_t, 0);
		debug_printf("OK. \n");


	//*****数字認識*****
		debug_printf("number_search ...");
		number_search(binr_t, &number);
		debug_printf("OK. \n");

		//8と認識された数字に対して、縦横比が 0.3 より小さい場合は 1 とする。
		if( number == 8 && ( (float)cvsize.width / (float)cvsize.height < 0.3) ){
			debug_printf("ROI捜査では8と認識されましたが、縦横比が %5.2f であるため 1 と判定します。\n",  (float)cvsize.width / (float)cvsize.height );
			number=1;
		}

		//printf("cvsize.width = %d\n",cvsize.width);
		//printf("cvsize.height = %d\n",cvsize.height);

		debug_printf("number=%d\n", number);

		if(number >= 0 && number < 10){
			if(count==0){
				x0_1=xy[k][0];
				num_1=number;
				count++;
			}else{
				x0_2=xy[k][0];
				num_2=number;
				count++;
				break; // ２個数字が認識されたら break する
			}
		}

		usleep(100000); // 0.1秒間隔を置く

	}

	// ２個の数字が認識された場合のみ、左右の数字を入れる
	if(count == 2){
		if(x0_1 < x0_2){
			number_l = num_1;
			number_r = num_2;
		}else{
			number_l = num_2;
			number_r = num_1;
		}


		debug_printf("left=%d,\tright=%d\n", number_l, number_r);
	}

	cvReleaseImage(&input);
	cvReleaseImage(&binr);
	cvReleaseImage(&trim);
	cvReleaseImage(&binr_t);

	return 0;
}

void number_search(IplImage *src_l, int* num_l){
	int f1l,f2l,f3l,f4l,f5l,f6l,f7l,f8l;	
	f1l = f2l = f3l = f4l = f5l = f6l = f7l = f8l = 0;
	int num_wl=0;
	int num_hl=0;
	int x, y, ans = -1;
	uchar col = 0;
	IplImage *targetImageL;


	targetImageL = cvCreateImage(cvGetSize(src_l), IPL_DEPTH_8U, 1);
	targetImageL = cvCloneImage(src_l);
	num_wl = src_l->width;
	num_hl = src_l->height;

//ROI走査(0-9の判定):left 1:二値化した画像の白部分 0:二値化した画像の黒部分
	for(x = 0; x < num_wl / 2; x++){
		col = targetImageL->imageData[targetImageL->widthStep * (num_hl / 4) + x];
		if(col == 255){
			f1l = 1;
			break;
		}
	}
	for(x = num_wl - 1; x >= num_wl - num_wl / 2; x--){
		col = targetImageL->imageData[targetImageL->widthStep * (num_hl / 4) + x];
		if(col == 255){
			f2l = 1;
			break;
		}
	}
	for(x = 0; x < num_wl / 2; x++){
		col = targetImageL->imageData[targetImageL->widthStep * (3 * num_hl / 4) + x];
		if(col == 255){
			f3l = 1;
			break;
		}
	}
	for(x = num_wl - 1; x >= num_wl - num_wl / 2; x--){
		col = targetImageL->imageData[targetImageL->widthStep*(3 * num_hl / 4) + x];
		if(col == 255){
			f4l = 1;
			break;
		}
	}
	for(y = 0; y < num_hl / 4; y++){
		col = targetImageL->imageData[targetImageL->widthStep * y + (num_wl / 2)];
		if(col == 255){
			f5l = 1;
			break;
		}
	}
	for(y = num_hl - 1; y >= num_hl - num_hl / 4; y--){
		col = targetImageL->imageData[targetImageL->widthStep * y + (num_wl / 2)];
		if(col == 255){
			f6l = 1;
			break;
		}
	}
	for(y = num_hl / 4; y < 3 * num_hl / 4; y++){
		col = targetImageL->imageData[targetImageL->widthStep * y + (num_wl / 2)];
		if(col == 255){
			f7l = 1;
			break;
		}
	}
	//1と8の見分け
	//col = targetImageL->imageData[targetImageL->widthStep * num_hl / 4 + (num_wl / 2)];
	col = targetImageL->imageData[targetImageL->widthStep * num_hl * 3/ 4 + (num_wl / 2)];
	if(col == 255){
		f8l = 1;
	}

	if(f7l == 0 && (f1l&f2l&f3l&f4l&f5l&f6l) == 1)		ans=0;
	else if((f1l|f4l)==0 && (f2l&f3l&f5l&f6l&f7l)==1)	ans=2;
	else if((f1l|f3l)==0 && (f2l&f4l&f5l&f6l&f7l)==1)	ans=3;
	else if((f3l|f5l|f6l)==0 && (f1l&f2l&f4l&f7l)==1)	ans=4;
	else if((f2l|f3l)==0 && (f1l&f4l&f5l&f6l&f7l)==1)	ans=5;
	else if(f2l==0 && (f1l&f3l&f4l&f5l&f6l&f7l)==1)		ans=6;
	else if((f3l|f6l|f7l)==0 && (f1l&f2l&f4l&f5l)==1)	ans=7;
	else if(f3l==0 && (f1l&f2l&f4l&f5l&f6l&f7l)==1)		ans=9;
	else if(f8l==0 && (f1l&f2l&f3l&f4l&f5l&f6l&f7l)==1)	ans=8;
	else if((f1l&f2l&f3l&f4l&f5l&f6l&f7l&f8l)==1)		ans=1;

	*num_l=ans;

	cvReleaseImage(&targetImageL);
}

void triming(IplImage *src, CvRect rect){
	cvSetImageROI(src,rect);
#if CV_VERSION_NUM == 1
	cvSaveImage("./img/trim.jpg", src);
#else
	cvSaveImage("./img/trim.jpg", src, NULL);
#endif
	cvResetImageROI(src);
}
}
