#ifndef INCLUDED_RUN_H
#define INCLUDED_RUN_H

#include <math.h>
#include <stdio.h>
#include "Debug.h"
#include "Motor.h"
#include "Uss.h"

class MotorController{
public:
	MotorController(const char* l_motor_path,const  char* r_motor_path, double first_pos_x=0.0, double first_pos_y=0.0, double first_theta_rad = M_PI/2.0, double mirs_radius=152); // constructor
	~MotorController(); // destructor

	double getThetaRad();
	double getThetaDeg();
	double getPosX();
	double getPosY();

	bool isFdError(); //ファイルディスクリプタが正しいか判定する.
	void initialize(); //走行関係の初期化.走行用のwhile文(while(!mc.straight(...));等)の前で呼ぶ.
	void setStartXY(double start_x,double start_y); //straightKeepDegのときの開始点を指定できる.通常は,initialize()したときの点に設定されている.

	//走行系関数. while(!mc.straight(...));のようにして呼ぶ.
	//止まりたくなければ,destに十分大きな値を指定する.
	//speed_lowerが大きすぎると,モータのスピードの上限に達して,正しい比で出力ができない可能性があるので注意.
	//accomplishment_valueで目標値達成の閾値を指定できる.P制御のための定数は,理解せずに使うことが無いようにデフォルト引数を指定していない.
	//
	//直進.引数はスピード、目標値.戻り値は、目標値を達成しているときに1、それ以外は0.
	int straight(double speed,double dest, double accomplishment_value=30);
	//直進.角度を一定にする補正付き.
	//MIRSの向きと,開始点から現在の点を見たときの向きを,同時にP制御している.
	int straightKeepDeg(double speed,double dest,double desire_deg,double Ktheta,double Kroute, double accomplishment_value=30);
	//直進.左の壁からの距離を一定にする補正付き.uss_l_valueにはwhile文の中で変化するように,距離の値を渡す.最適化には注意.
	int straightKeepUss(double speed,double dest, double uss_l_value, double desire_uss_l_value, double Kuss, double accomplishment_value=30);
	//回転.引数はスピード,目標値.戻り値は,目標値を達成しているときに1,それ以外は0.
	int rotate(double speed,double deg, double accomplishment_value=30);
	//旋回.引数はスピード, 回転する角度, 半径, 時計回りor半時計回り(1 or -1)
	int circle(double speed_lower, double deg, double radius, int rot_direction, double accomplishment_value=30);
	//ブレーキ.スピード指定値を0にする.走行系関数は基本的にbrake()しなければ止まらない.
	void brake();

private:
	double rotaryEncoderValueL,rotaryEncoderValueR; //clearしてからの左右それぞれの移動距離
	double oldRotaryEncoderValueR,oldRotaryEncoderValueL;
	double posX,posY; //現在位置[mm]
	double theta; //現在角度[rad]
	double startX,startY;
	static int idCounter;
	int id; //オブジェクトの識別番号
	double mirs_radius; //MIRSの半径
	double diffPrev; //runPidV()に使用
	double diffSum;
	Motor *lMotorp; //モータのオブジェクトへのポインタ.ioerrorRecovery()のためにポインタを使用している.
	Motor *rMotorp;
	char lMotorPath[64]; //開き直すときに使用
	char rMotorPath[64];

	void start(); //start()とstop()の間で走行関数を呼ぶ
	void stop();
	void update(); //ロータリーエンコーダ値の更新と位置計算
	void ioerrorRecovery();
	double runPidV(double diff,double kp,double kd,double ki); //速度補正値をPIDで計算する.引数：左右の走行距離の差、戻り値：速度補正値
	static char sign(double num);
	double controlSpeedRange(double speed);

	//引数はスピード,左右の目標距離,角度補正のON/OFF,目標角度,角度を一定に保つための定数,直進軌道から外れないようにするための定数
	//角度補正は,角度を指定して,その角度を保つように制御する.角度補正するときは事前にsetStartXY()を呼ぶ.
	//直進しない時はkeep_deg_flagはfalseのままにする.
	int std(double speed_lower, double dest_l, double dest_r, double accomplishment_value=30, bool keep_deg_flag=false, double deg=90, double Ktheta=0.25, double Kroute=0.0, bool keep_uss_flag=false, double uss_l_desire_value=30, double uss_l_value=30, double Kuss=0.01);
};
#endif
