#include "TestTalk.h"
MotorController g_mc(MOTOR_L_PATH, MOTOR_R_PATH, MIRS_RADIUS, 0, 0, M_PI/2.0);
Uss g_uss(S_PORT_PATH, USS_NUM);
IoRead g_ir(IO_PATH);

int main(void/*int argc, char *argv[]*/){
	using namespace std;

	//USS スレッド生成
	cout << "USS thread start ...";
	g_uss.startThread();

	Talk::repl();

	//スレッドのキャンセル
	cout << "USS thread finish ...";
	g_uss.finishThread();

	return 0;
}

namespace Talk{
using namespace std;
void repl(){
	string command("");
	//ファイルディスクリプタが初期化できているかの確認
	if(g_mc.isFdError() || g_uss.isFdError() || g_ir.isFdError()){
		cerr << "main() : fd couldn't open\n";
		return;
	}
	cout << "start Talk\n";
	while(!Talk::print(Talk::eval(Talk::read(command))));
}

string read(string command){
	cout << ">";
	cin >> command;
	return command;
}

string exit(){
	return "bye.";
}

string help(string command){
	if(command=="move"){
		cout << "straight - 直進します" << endl;
		cout << "rotate - その場で回ります" << endl;
		cout << "circle - 円を描いて回ります" << endl;
		cout << "return - コマンドの入力状況を元に戻します" << endl;
		cout << "help - 助けます" << endl;
		cout << "exit - 終わります" << endl;
		return move();
	}else if(command=="view"){
		cout << "uss - 超音波距離センサモジュールの出力を表示します" << endl;
		cout << "touch - タッチセンサの出力を表示します" << endl;
		cout << "position - MIRSの現在の位置と角度（推定）を表示します" << endl;
		cout << "ball - ボールまでの距離と角度のずれを表示します" << endl;
		cout << "return - コマンドの入力状況を元に戻します" << endl;
		cout << "help - 助けます" << endl;
		return view();
	}else{
		cout << "move - 動きます" << endl;
		cout << "stop - 止まります" << endl;
		cout << "view - 状態を表示します" << endl;
		cout << "return - コマンドの入力状況を元に戻します" << endl;
		cout << "help - 助けます" << endl;
		cout << "exit - 終わります" << endl;
		cout << "[Ctrl+C] - 中断します" << endl;
	}
	return "ok.";
}

string move_straight(){
	string command;
	cout << "straight distance [cm]>";
	cin >> command;
	if(command=="return") return "ok.";
	g_mc.initialize();
	double deg=g_mc.getThetaDeg();
	while(!g_mc.straightKeepDeg(STRAIGHT_SPEED,10*atoi(command.c_str()),deg,0.1,0));
	g_mc.brake();
	return "ok.";
}

string move_straight_uss(){
	string command;
	cout << "straight distance [cm]>";
	cin >> command;
	if(command=="return") return "ok.";
	g_mc.initialize();
	while(!g_mc.straightKeepUss(STRAIGHT_SPEED,10*atoi(command.c_str()),g_uss.getValue(USS_L),30,2.0));
	g_mc.brake();
	return "ok.";
}

string move_rotate(){
	string command;
	cout << "rotate angle [deg]>";
	cin >> command;
	if(command=="return") return "ok.";
	g_mc.initialize();
	while(!g_mc.rotate(ROTATE_SPEED,atoi(command.c_str())));
	g_mc.brake();
	return "ok.";
}

string move_circle(){
	int value[3];
	string command;
	cout << "circle direction(clockwise or counterclockwise)>";
	cin >> command;
	if(command=="return") return "ok.";
	value[0] = (command=="clockwise")?1:-1;
	cout << "circle radius [cm]>";
	cin >> command;
	if(command=="return") return "ok.";
	value[1] = atoi(command.c_str());
	cout << "circle angle [deg]>";
	cin >> command;
	if(command=="return") return "ok.";
	value[2] = atoi(command.c_str());
	g_mc.initialize();
	while(!g_mc.circle(CIRCLE_SPEED,value[2],10*value[1],value[0]));
	return "ok.";
}

string move(){
	string command;
	cout << "move>";
	cin >> command;
	if(command=="help"){
		command = help("move");
	}else if(command=="straight" || command=="st"){	command = move_straight();
	}else if(command=="straight_uss" || command=="st_uss" || command == "stu"){	command = move_straight_uss();
	}else if(command=="rotate" || command=="rt"){	command = move_rotate();
	}else if(command=="circle" || command=="cc"){	command = move_circle();
	}else if(command=="return"){	command = "ok.";
	}else{
		cout << "unnkown command" << endl;
		command="NG.";
	}
	return command;
}

string view(){
	string command;
	cout << "view>";
	cin >> command;
	if(command=="help"){
		command = help("view");
	}else if(command=="uss"){
		cout << "uss left  : " << g_uss.getValue(0) << endl;
		cout << "uss front : " << g_uss.getValue(1) << endl;
		command="ok.";
	}else if(command=="touch"){
		cout << "touch left   : " << g_ir.get(TOUCH_L) << endl;
		cout << "touch center : " << g_ir.get(TOUCH_C) << endl;
		cout << "touch right  : " << g_ir.get(TOUCH_R) << endl;
		command="ok.";
	}else if(command=="position"){
		cout << "position x : " << g_mc.getPosX() << endl;
		cout << "position y : " << g_mc.getPosY() << endl;
		cout << "angle      : " << g_mc.getThetaDeg() << endl;
		command="ok.";
	}else if(command=="ball"){
		double dis,deg;
		Ball::ball_detect(&dis,&deg);
		if(dis==-999 && deg==-999){
			cout << "ball is nothing." << endl;
			command="NG.";
		}else{
			cout << "ball distance [cm]  : " << dis << endl;
			cout << "ball angle    [deg] : " << deg << endl;
			cout << "ball position(x) [cm]  : " << g_mc.getPosX()+dis*cos(deg/360.0*M_PI) << endl;
			cout << "ball position(y) [cm]  : " << g_mc.getPosY()+dis*sin(deg/360.0*M_PI) << endl;
			command="ok.";
		}
	}else if(command=="return"){
		command="ok.";
	}else{
		cout << "unnkown command" << endl;
		command="NG.";
	}
	return command;
}

string eval(string command){
	if(command == "exit"){
		command = exit();
	}else if(command == "help"){
		command = help("");
	}else if(command == "move" || command == "mv"){
		command = move();
	}else if(command == "stop"){
		g_mc.brake();
		command="ok.";
	}else if(command == "view"){		
		command = view();
	}else if(command == "return"){
		command = "ok.";
	}else{
		command = "repl command error";
	}
	return command;
}

int print(string command){
	cout << command << endl;
	if(command=="bye."){
		return 1;
	}
	return 0;	
}
} /*namespace Talk*/
