/**********************
serial port setting
**********************/
#include <string.h>
#include "SerialFunc.h"
 
/*----- Setting serial port method -----*/
int setSerialPort(int fd, struct termios *oldtio){

    struct termios newtio;

    /* Open modem device for reading and writing and not as controlling tty */
    /* 読み書きの為にモデムデバイスをオープンする。ノイズによってCTRL-Cが
              たまたま発生しても接続が切れないようにtty制御はしない */

    tcgetattr(fd,oldtio); /*現在の設定をget*/

    /* Clear the struct for new port settings */ 
    /* 制御文字の初期化を行う 0は何のフラグもない==何も設定しない*/
    newtio.c_iflag = 0;
    newtio.c_oflag = 0;
    newtio.c_cflag = 0;
    newtio.c_lflag = 0;
    newtio.c_line = 0;
    memset(newtio.c_cc,0,sizeof(newtio.c_cc));

    /* Settings for new port
      BAUDRATE : Set bps rate. You could also use cfsetispeed and cfsetospeed.
      ボーレートの設定。cfsetispeed,cfsetospeedも使用できる
      CRTSCTS  : Output hardware flow control ( only used if the cable has all
                  necessary lines. See sect. 7 of Serial-HOWTO )
             出力のハードウェアフロー制御(必要な結線が全てされているケースのみ)
      CS8      : 8n1 ( 8bit,no parity,1 stopbit)
      CLOCAL   : local connection,no modem control
      CREAD    : enable receiving characters
                    受信文字を有効にする
    */
    newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD ;

    /* IGNPAR  : ignore bytes with parity errors */
    /* (パリティエラーのデータは無視する )*/
    newtio.c_iflag = IGNPAR;

    newtio.c_oflag = 0;     /* Raw output (Rawモードでの出力)*/
    newtio.c_lflag = 0;     /* Set input mode (non-canonical,no echo,....) */
    newtio.c_cc[VTIME] = 0; /* inter-character timer 0 sec
                                 (キャラクタタイマを使わない) */
    newtio.c_cc[VMIN] = 1;  /* read single char received
                                 (読み込みをブロックする 一文字ずつ読み込む) */ 

    /* Now clean the modem line and activate the settings for the port */
    /* モデムラインをクリアする */
    tcflush(fd,TCIFLUSH);
    /* 新しい設定を適用する */
    tcsetattr(fd,TCSANOW,&newtio);

    return 0;
}

/*----- Close serial port method -----*/
void closeSerialPort(int fd, struct termios *oldtio){
    tcsetattr(fd, TCSANOW, oldtio); /* Restore the old port settings */
}

/*----- Output a character to current serial port -----*/
int putSerialChar(int fd, char c){
    if(write(fd,&c,1) != 1)
        return -1;
    return 0;
}

/*----- Output a string to current serial port -----*/
int putSerialString(int fd, char *s){
    if(write(fd,s,strlen(s)) != 1)
        return -1;
    return 0;
}

/*----- Get a character from current serial port -----*/
int getSerialChar(int fd){
    char c;
    int res, rval;
    fd_set readfs;
    struct timeval Timeout;
    Timeout.tv_usec=50000;
    Timeout.tv_sec=0;
    FD_ZERO(&readfs);
    FD_SET(fd,&readfs);
    res = select((fd + 1),&readfs,NULL,NULL,&Timeout);
    if(res==0){
    	c=0;
    	return c;
    }
    rval = read(fd,&c,1);
    if(rval == -1){
        perror("uss getSerialChar");
    }
    return ((unsigned char)c);
}
