#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <termios.h>
#include <pthread.h>
#include <stdint.h>
#include <fcntl.h>

#include "Uss.h"
#include "SerialFunc.h"

bool Uss::threadRunning=false;
int Uss::idCounter=0;

void* Uss::g_readThread(void* args){
	Uss* ussp = (Uss*)args;
	while(1){
		for(int i=0; i < ussp->getNum(); i++){
			pthread_testcancel();
			if( (ussp->setValue(i, ussp->read(i)) ) == -1 ){
				fprintf(stderr,"Uss readThread error : value is -1\n");
			}
			//debug_printf("uss[%d] : %d\n", i, ussp->getValue(i));
			usleep(38*1000); //隣り合うUSSが干渉するため30msec以上waitを入れる。
		}
	}
	return NULL;
}

Uss::Uss(const char *fd_path, char num){
	this->id = idCounter;
	idCounter++;
	this->fd=open(fd_path,O_RDWR | O_NOCTTY | O_NONBLOCK); //USSシリアルポートのオープン
	if(this->isFdError()){
		fprintf(stderr,"Uss id:%d fd0 open error\n",this->id);
	}
	value = new int(num);
	for(int i = 0; i < num; i++){
		value[i] = 0;
	}
	this->num = num;
}

Uss::~Uss(){
	printf("deleting Uss id:%d ...",this->id);
	if(!this->isFdError()){
		close(this->fd);
		printf("OK.\n");
	}else{
		printf("NG.\n");
		fprintf(stderr,"\t Uss id:%d fd < 0\n",this->id);
	}
	if(value!=NULL){
		delete value;
	}
}

bool Uss::isFdError(){
	return this->fd < 0;
}

char Uss::getNum(){
	return this->num;
}

int Uss::getValue(int uss_num){
	int tmp=this->value[uss_num];
	return (tmp==9999)? 300 : tmp;
}

int Uss::setValue(int uss_num, int new_value){
	return this->value[uss_num] = new_value;
}

//引数で指定したUSS番号(0~ussMax)の値を読み込み。戻り値：読み取り値(cm)
int Uss::read(int uss_num){
	unsigned char rs_rbuffer[2];
	unsigned char rs_sbuffer[1];
	int data = 99999;
	struct termios oldtio;
	if(uss_num < 0 || uss_num >= this->num){
		fprintf(stderr,"uss_read error: uss_num is bad.\n");
		return -1;
	}
	rs_sbuffer[0] = uss_num+1;

	if(this->isFdError()){
		fprintf(stderr,"uss read: fd0 open error\n");
		return -1;
	}

	setSerialPort(fd,&oldtio); /*毎回やる必要はない？*/
	putSerialString(fd,(char*)rs_sbuffer);

	rs_rbuffer[0] = getSerialChar(this->fd);
	rs_rbuffer[1] = getSerialChar(this->fd);
	if(rs_rbuffer[1]==0xff){
		fprintf(stderr,"uss read: 0xff.\n");
		return -1;
	}
	data = (rs_rbuffer[1]<<8) + rs_rbuffer[0] + 20;

	closeSerialPort(fd,&oldtio); /*毎回やる必要はない？*/
	return data;
}

void Uss::startThread(){
	if(pthread_create( &(this->uss_task), NULL, g_readThread , (void *)this ) != 0){
		printf("NG.\n");
		fprintf(stderr,"uss thread create error\n");
	}else{
		printf("OK. uss thread created.\n");
		threadRunning=true;
	}
}

void Uss::finishThread(){
	if(threadRunning){
		pthread_cancel(this->uss_task);
		pthread_detach(this->uss_task);
		threadRunning=false;
		printf("OK. uss thread deleted.\n");
	}else{
		printf("NG.\n");
		fprintf(stderr, "uss thread is not running.\n");
	}
}

