#!/usr/bin/env python
# -*- coding: utf-8 -*-
#import cv
from cv2 import cv
import math, position

class BallDetector(object):
    BALL2R          = 200 #ボールの直径
    CAM_VISIBLE_DEG = 60  #カメラの視野角
    DP              = 2
    HOUGH_PARAM1    = 150 #Cannyのエッジ検出器で用いる二つの閾値の高い方の値
    HOUGH_PARAM2    = 50  #中心検出計算時の閾値．小さすぎると誤検出が多くなる．これに対応する値が大きい円から順に検出される．
    CANNY_PARAM2    = 100
    SMOOTH_PARAM1   = 11   #アパーチャの幅．正の奇数 (1, 3, 5, ...) でなければいけません．
    #THRESHOLD_MIN = cv.Scalar(168,60,60)   #HSVの閾値。
    #THRESHOLD_MAX = cv.Scalar(12,255,255) #HSVの閾値。
    THRESHOLD_MIN = cv.Scalar(95,60,70)   #HSVの閾値。
    THRESHOLD_MAX = cv.Scalar(115,255,255) #HSVの閾値。
    def __init__(self,height,width,identification,debug=False,windowdebug=False):
        self.tmp3c = cv.CreateImage((width,height),cv.IPL_DEPTH_8U,3)
        self.tmp1c = cv.CreateImage((width,height),cv.IPL_DEPTH_8U,1)
        self.tmp1c_sub = cv.CreateImage((width,height),cv.IPL_DEPTH_8U,1)
        self.output1c = cv.CreateImage((width,height),cv.IPL_DEPTH_8U,1)
        self.deg = -1
        self.distance = -1
        self.identification = identification
        self.debug = debug
        self.windowdebug = windowdebug
        self.almost_white_flag = False
        if self.windowdebug == True:
            cv.NamedWindow("input"+str(identification))
            cv.NamedWindow("bin"+str(identification))
            cv.NamedWindow("denoise"+str(identification))
            cv.NamedWindow("smooth"+str(identification))

    def update(self,input3c):
        if self.windowdebug == True:
            cv.ShowImage("input"+str(self.identification),input3c)
            cv.WaitKey(10)
        self._binarization(input3c,self.output1c)
        if self.windowdebug == True:
            cv.ShowImage("bin"+str(self.identification),self.output1c)
            cv.WaitKey(10)
        self._denoise(self.output1c)
        if self.windowdebug == True:
            cv.ShowImage("denoise"+str(self.identification),self.output1c)
            cv.WaitKey(10)
        #hist = cv.CreateHist([128],cv.CV_HIST_ARRAY,[[0,256]],1)
        hist = cv.CreateHist([2],cv.CV_HIST_ARRAY)
        cv.CalcHist([self.output1c],hist)
        if cv.GetMinMaxHistValue(hist)[2][0] == 0:
             self.almost_white_flag = True
        else:
             self.almost_white_flag = False
        print self.almost_white_flag
        self._smooth(self.output1c)
        if self.windowdebug == True:
            cv.ShowImage("smooth"+str(self.identification),self.output1c)
            cv.WaitKey(10)
        self.distance,self.deg = self._detectCircles(self.output1c)
        print "balldetector update : " + str((self.distance,self.deg))
    def clear(self):
        self.distance,self.deg = (-1,-1)
    def isBall(self):
        if self.distance <= 0: return False
        else: return True

    def getRelativePosition(self,camera_position):
        if self.distance == -1 : return (-1,-1)
        theta = self.deg/180.0*math.pi
        print camera_position,self.distance,self.deg
        x = camera_position.x + self.distance*math.cos(camera_position.getThetaRad() + theta)
        y = camera_position.y + self.distance*math.sin(camera_position.getThetaRad() + theta)
        return position.Position(x,y)

    def _binarization(self,src3c,dst1c):
        cv.CvtColor(src3c,self.tmp3c,cv.CV_BGR2HSV)
        two_area_flag = False
        rev_flag = [self.THRESHOLD_MIN[i] > self.THRESHOLD_MAX[i] for i in range(3)]
        threshold_min1 = []
        threshold_max1 = []
        threshold_min2 = []
        threshold_max2 = []
        if True in rev_flag:
            for i in range(3):
                if rev_flag[i]:
                    threshold_min1.append(0)
                    threshold_max1.append(self.THRESHOLD_MAX[i])
                    threshold_min2.append(self.THRESHOLD_MIN[i])
                    threshold_max2.append(180)
                    two_area_flag = True
                else:
                    threshold_min1.append(self.THRESHOLD_MIN[i])
                    threshold_max1.append(self.THRESHOLD_MAX[i])
                    threshold_min2.append(self.THRESHOLD_MIN[i])
                    threshold_max2.append(self.THRESHOLD_MAX[i])
            threshold_min1.append(0)
            threshold_max1.append(0)
            threshold_min2.append(0)
            threshold_max2.append(0)
            cv.InRangeS(self.tmp3c,threshold_min1,threshold_max1,self.tmp1c)
            cv.InRangeS(self.tmp3c,threshold_min2,threshold_max2,self.tmp1c_sub)
            cv.Or(self.tmp1c,self.tmp1c_sub,dst1c)
        else:
            cv.InRangeS(self.tmp3c,self.THRESHOLD_MIN,self.THRESHOLD_MAX,dst1c)
        self._output(dst1c,"binarization")
    def _denoise(self,src1c):
        cv.Dilate(src1c, self.tmp1c, iterations=2)
        self._output(self.tmp1c,"denoise_1Dilate")
        cv.Erode(self.tmp1c, src1c, iterations=2)
        self._output(src1c,"denoise_2Erode")
        cv.Erode(src1c, self.tmp1c, iterations=4)
        self._output(self.tmp1c,"denoise_3Erode")
        cv.Dilate(self.tmp1c, src1c, iterations=4)
        self._output(src1c,"denoise_4Dilate")
    def _smooth(self,src1c):
        #cv.Canny(src1c,self.tmp1c,self.CANNY_PARAM2/2,self.CANNY_PARAM2)
        cv.Copy(src1c,self.tmp1c)
        self._output(self.tmp1c,"canny")
        cv.Smooth(self.tmp1c, src1c, param1=self.SMOOTH_PARAM1)
        self._output(src1c,"smooth")
        if self.debug == True: 
            cv.Canny(src1c,self.tmp1c,self.CANNY_PARAM2/2,self.CANNY_PARAM2)
            self._output(self.tmp1c,"canny2")
    def _detectCircles(self,src1c):
        data = cv.CreateMat(256,1,cv.CV_32FC3)
        cv.HoughCircles(src1c,data,cv.CV_HOUGH_GRADIENT, self.DP, self.output1c.height/4, self.HOUGH_PARAM1, self.HOUGH_PARAM2)
        for i in range(data.rows):
            x,y,radius = data[i,0]
            cv.Circle(src1c, (cv.Round(x), cv.Round(y)), cv.Round(radius), cv.CV_RGB(0,0,0xff),2)
        self._output(src1c,"detectCircles")
        if data.rows == 1:
            distance = self.BALL2R * src1c.width/2 * math.sqrt(3) / radius
            deg = self.CAM_VISIBLE_DEG * ( src1c.width/2 - x ) / src1c.width
            return (distance,deg)
        else:
            return (-1,-1)
    def _output(self,out,identification):
        if self.debug == True:
            cv.SaveImage("data/outputnum"+str(self.identification)+"("+ identification +").png",out)
