#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""カメラの情報および、ボール検出、数字検出を扱うモジュール"""
import cv, balldetector, numberdetector, threading, time

class Camera(object):
    def __init__(self, position, index, debug=True, size=(120,160)):
        self.position = position
        self.index = index
        height, width = size
        self.ball_detector = balldetector.BallDetector(height=height,width=width,identification=index,debug=debug)
        self.number_detector = numberdetector.NumberDetector(height=height,width=width,identification=index,debug=debug)
        self.input3c = cv.CreateImage((width,height),cv.IPL_DEPTH_8U,3)
        dummy = cv.CaptureFromCAM(self.index)
        cv.GrabFrame(dummy)
        del dummy
        self.cap = cv.CaptureFromCAM(self.index) #このデータが寿命を迎えると、ここから取得した画像がセグメンテーション違反を起こす。1つまでしかデバイスを開けないので注意
        cv.SetCaptureProperty(self.cap,cv.CV_CAP_PROP_FRAME_WIDTH,width)
        cv.SetCaptureProperty(self.cap,cv.CV_CAP_PROP_FRAME_HEIGHT,height)
    def _loadImageFile(self):
        for i in range(4): cv.GrabFrame(self.cap) #バッファリングされているために、飛ばさないとリアルタイム性がなくなる
        self.input3c = cv.QueryFrame(self.cap)
        #cv.ShowImage("debug"+str(self.index),self.input3c)
        #cv.WaitKey(5)
        assert self.input3c != None, "camera is not set, or wrong permission error"
    def precapture(self):
        for i in range(5): cv.QueryFrame(self.cap) #適当に捨てる
    def getBallRelativePosition(self):
        ret = self.ball_detector.getRelativePosition(self.position)
        if self.ball_detector.almost_white_flag == True:
            print "almost while"
            ret = self.position
        return ret 
    def isBall(self):
        return self.ball_detector.isBall()
    def clear(self):
        self.ball_detector.clear()
        self.number_detector.clear()
    def getNumbers(self):
        return self.number_detector.numbers
    def update_ball(self):
        self._loadImageFile()
        self.ball_detector.update(self.input3c)
    def update_number(self):
        self._loadImageFile()
        self.number_detector.update(self.input3c)
    def snapshot(self):
        print "save : " + "data/snapshot"+str(self.index)+".jpg"
        self.precapture()
        cv.SaveImage("data/snapshot"+str(self.index)+".jpg",cv.QueryFrame(self.cap))
