#!/usr/bin/env python
# -*- coding: utf-8 -*-
import time, math, threading
import position, copy, hardwareconst

class ControllerThread(threading.Thread):
    def __init__(self, ussthread, motorcontroller, image, fpgaio, field, patrol_route, search_route, datasender):
        threading.Thread.__init__(self)
        self.ussthread       = ussthread
        self.motorcontroller = motorcontroller
        self.image           = image
        self.fpgaio          = fpgaio
        self.field           = field
        self.patrol_route    = patrol_route
        self.search_route    = search_route
        self.datasender = datasender
        self.mode = "patrol"
        self.wall_flag = {"r":False, "c":False, "l":False}
        self.mainthread = threading.currentThread()
        self.thread = threading.Thread(target=self.run)
    def start(self):
        threading.Thread.start(self)

    def checkWall(self):
        return dict([(key, self.fpgaio.get(hardwareconst.TOUCH[key]) == 1) for key in hardwareconst.TOUCH.keys()])

    def rotateToPosition(self, endpos, startpos,complete_delta_deg=10):
        """開始座標(現在地)と目的座標を引数に取り、目的座標を向くように回転する。"""
        #開始座標から目的座標に向かうベクトルを求めて、直進時の角度を計算する。
        print "rotate from " + str(startpos) + " to " + str(endpos)
        vector_to_endpos = endpos - startpos
        deg_absolute = vector_to_endpos.getArgDeg()
        
        #目的座標に向くように回転する。
        self.motorcontroller.setRotateAbsolute(speed_abs=5,deg_absolute=deg_absolute,kparam_d=0.01,complete_delta_deg=complete_delta_deg) #kparam_dが大きいとおかしくなる。PID制御を見直す必要がある
        while (not self.motorcontroller.isComplete()) and self.mainthread.isAlive():
            self.motorcontroller.run()
        self.motorcontroller.brake()

    def straightToPosition(self,endpos,startpos):
        """開始座標(現在地)と目的座標を引数に取り、目的座標まで直進する。障害物を検知したらフラグをセットして中断する。"""
        #開始座標から目的座標に向かうベクトルを求めて、直進時の角度を計算する。
        print "straight from " + str(startpos) + " to " + str(endpos)
        vector_to_endpos = endpos - startpos
        startposcopy = copy.deepcopy(startpos)
        startposcopy.theta = vector_to_endpos.getArgRad()
        
        #目的座標に向かって直進。
        #衝突を検知した場合、フラグを立てる
        #実機で動いた時のパラメータ(ver1.0時)kparam_d=0.005,kparam_p=0.01,kparam_i=0.04
        #今のD,I制御は大きく働きすぎる可能性がある。
        self.motorcontroller.setStraightPID(speed_abs=10,dest=abs(vector_to_endpos),startpos=startposcopy,kparam_d=0.0,kparam_p=0.6,kparam_i=0.01,complete_delta=30)
        while not self.motorcontroller.isComplete() and self.mainthread.isAlive():
            self.motorcontroller.run()
            
            #壁検知フラグをセット、検知しなければループ続行。
            self.wall_flag = self.checkWall()
            if True in self.wall_flag.values(): break
        self.motorcontroller.brake()

    def backToPosition(self, endpos, startpos):
        """開始座標(現在地)と目的座標を引数に取り、目的座標までバックする。"""
        #開始座標から目的座標に向かうベクトルを求めて、直進時の角度を計算する。
        vector_to_endpos = endpos-startpos
        startposcopy = copy.deepcopy(startpos)
        startposcopy.theta = vector_to_endpos.getArgRad() + math.pi #バックするので、進行方向とは逆を向く
        print "back to :" + str(endpos)
        
        #開始座標から目的座標にバックする。
        self.motorcontroller.setStraightPID(speed_abs=10,dest=-abs(vector_to_endpos),startpos=startposcopy,kparam_d=0.01,kparam_p=0,kparam_i=0,complete_delta=20)
        while not self.motorcontroller.isComplete() and self.mainthread.isAlive():
            self.motorcontroller.run()
            #print self.motorcontroller
        self.motorcontroller.brake()

    def position_compensate(self):
        """バックして位置情報を補正する"""
        if   self.wall_flag["l"] == True:
            self.backToPosition(endpos=self.oldpos, startpos=self.motorcontroller.pos)
            self.motorcontroller.pos.theta += 10.0/360.0*math.pi
        elif self.wall_flag["r"] == True:
            self.backToPosition(endpos=self.oldpos, startpos=self.motorcontroller.pos)
            self.motorcontroller.pos.theta -= 10.0/360.0*math.pi
        elif self.wall_flag["c"] == True: 
            self.motorcontroller.pos.x += 50.0 * math.cos(self.motorcontroller.pos.theta)
            self.motorcontroller.pos.y += 50.0 * math.sin(self.motorcontroller.pos.theta)
            self.backToPosition(endpos=self.oldpos, startpos=self.motorcontroller.pos)

    def check_treasure(self):
        checkpoint_data = self.field.getNearestObject("checkpoint",self.oldpos,0)
        if checkpoint_data != []:
            print("checkpoint"+str((checkpoint_data[2],checkpoint_data[3])))
            self.rotateToPosition(endpos=position.Position(checkpoint_data[2],checkpoint_data[3]), startpos=self.oldpos)
            """
            self.image.start(mode="number")
            safety_counter=0
            while self.mainthread.isAlive():
                numbers = self.image.getNumbers()
                time.sleep(0.5) #画像処理が終わるように適当に設定する。開発期間があればこの時間に少し回転、前後移動などを入れて、改善する
                safety_counter+=1
                if len(numbers) == 2 or safety_counter>=10:
                    self.image.stop()
                    print "number detect : "+str(numbers)
                    break
            """
            self.image.snapshot()
            for i in range(len(checkpoint_data[4:])/2):
                sendfile = "snapshot"+str(checkpoint_data[4+2*i])+".jpg"
                sendfile_change = "treasure_"+str(checkpoint_data[5+2*i])+".jpg"
                print "sending" + sendfile_change
                if self.datasender != None:
                    self.datasender.send_file(sendfile,filename_change=sendfile_change,client_directory="data")

    def compensate_with_uss(self):
        if self.ussthread.values[hardwareconst.USS_POS["l"]] == 20:
            print "left wall is near."
            self.motorcontroller.pos.x += -10.0 * math.sin(self.motorcontroller.pos.theta)
            self.motorcontroller.pos.y += 10.0 * math.cos(self.motorcontroller.pos.theta)
        if self.ussthread.values[hardwareconst.USS_POS["r"]] == 20:
            print "right wall is near."
            self.motorcontroller.pos.x += 10.0 * math.sin(self.motorcontroller.pos.theta)
            self.motorcontroller.pos.y += -10.0 * math.cos(self.motorcontroller.pos.theta)
        if self.ussthread.values[hardwareconst.USS_POS["rf"]] == 20 or self.ussthread.values[hardwareconst.USS_POS["lf"]] == 20:
            print "front wall is near."
            self.motorcontroller.pos.x += 10.0 * math.cos(self.motorcontroller.pos.theta)
            self.motorcontroller.pos.y += 10.0 * math.sin(self.motorcontroller.pos.theta)
        

    def patrol(self):
        """巡回して、宝をチェックするモードのためのルーチン。"""
        self.fpgaio.put((0,0,0,1))
        self.oldpos = position.Position(0,0) #現在地を初期化
        already_checked = False
        while self.mainthread.isAlive():
            if not already_checked:
                self.check_treasure() #現在地がチェックポイントならば、宝のチェックを行う
            if self.patrol_route.isEnd(): #経路データが終わりならば、モードを探索にして終了。
                self.mode = "search"
                return

            self.nextpos = self.patrol_route.getPosition() #目的座標データを得る。
            print("goto : "+str((self.nextpos.x,self.nextpos.y))) #目的座標を表示
            self.rotateToPosition(  endpos=self.nextpos, startpos=self.motorcontroller.pos) #目的位置を向くように回転
            self.straightToPosition(endpos=self.nextpos, startpos=self.motorcontroller.pos) #目的位置に直進。障害物を検知したら中断される

            if True in self.wall_flag.values(): #壁検知フラグが立っていたら、もとの位置まで戻って補正する。
                self.position_compensate()
                already_checked = True
            else :
                self.patrol_route.next()
                self.oldpos  = copy.deepcopy(self.nextpos)
                already_checked = False

            self.compensate_with_uss()

    def search(self):
        """巡回しながら怪盗を探索するモードのためのルーチン"""
        self.fpgaio.put((0,0,1,0))
        print("search start")
        self.oldpos=position.Position(self.motorcontroller.pos.x,self.motorcontroller.pos.y) #現在地を初期化
        first = True
        while self.mainthread.isAlive():
            print "ball : " + str(self.image.isBall())
            self.image.start("ball")
            for i in range(10):
                if self.image.isBall(): #怪盗の認識(発見)。怪盗がいたら、追跡モードに遷移
                    print("found the phantom thief\n"+"search end")
                    self.mode = "chase"
                    return
                time.sleep(0.1)
            self.image.stop()

            if len(self.search_route) == 0:
                if first:
                    self.oldpos = copy.deepcopy(self.nextpos)
                    first = False
            elif len(self.search_route) == 1:
                if first:
                    if self.search_route.isEnd():
                        self.search_route.seek(0) #経路データが終わりならば、最初から探索し直す。
                    self.nextpos = self.search_route.getPosition() #目的座標データを得て、Pisitionクラスのデータに変換する。
                    print("goto : "+str((self.nextpos.x,self.nextpos.y))) #目的座標を表示
                    self.rotateToPosition(  endpos=self.nextpos, startpos=self.motorcontroller.pos, complete_delta_deg=5) #目的位置を向くように回転
                    self.straightToPosition(endpos=self.nextpos, startpos=self.motorcontroller.pos) #目的位置に直進。障害物を検知したら中断する

                    if True in self.wall_flag.values(): #壁検知フラグが立っていたら、もとの位置まで戻る。
                        self.position_compensate()
                    else :
                        self.search_route.next()
                        self.oldpos  = copy.deepcopy(self.nextpos)
                        first = False
            else:
                if self.search_route.isEnd():
                    self.search_route.seek(0) #経路データが終わりならば、最初から探索し直す。
                self.nextpos = self.search_route.getPosition() #目的座標データを得て、Pisitionクラスのデータに変換する。
                print("goto : "+str((self.nextpos.x,self.nextpos.y))) #目的座標を表示
                self.rotateToPosition(  endpos=self.nextpos, startpos=self.motorcontroller.pos) #目的位置を向くように回転
                self.straightToPosition(endpos=self.nextpos, startpos=self.motorcontroller.pos) #目的位置に直進。障害物を検知したら中断する

                if True in self.wall_flag.values(): #壁検知フラグが立っていたら、もとの位置まで戻る。
                    self.position_compensate()
                else :
                    self.search_route.next()
                    self.oldpos  = copy.deepcopy(self.nextpos)

    def chase(self):
        """見つけた怪盗を追跡するためモードのためのルーチン"""
        self.fpgaio.put((0,1,0,0))
        print("chase start")
        while self.mainthread.isAlive():
            #怪盗の場所を認識する。怪盗が見つからなくなったら、ルート補正に移行する
            count = 0
            while not self.image.isBall() :
                if count >= 4:
                    print("miss")
                    self.image.stop()
                    self.mode = "route_compensate"
                    return
                time.sleep(0.1)
                count += 1
            relative_position = self.image.getBallRelativePosition()
            print("phantom thief is :"+str(abs(relative_position)) +", "+ str(relative_position.getArgDeg()))

            deg = relative_position.getArgDeg()
            position.periodicRestrict(deg,-180,180)
            print "deg to thief "+str(deg-90)
            if abs(deg-90) > 20: #怪盗への相対角度が20度より大きい時は回転する
                print "let's rotate"
                self.motorcontroller.setRotate(speed_abs=5,deg=deg-90,kparam_d=0.01,complete_delta_deg=10)
                while not self.motorcontroller.isComplete() and self.mainthread.isAlive():
                    self.motorcontroller.run()
                self.motorcontroller.brake()
            else: #怪盗への相対角度が小さい時は直進を行う(角度補正付きなので、直進しながら怪盗を正面に捉えようとする)
                print "let's go straight"
                startpos = copy.deepcopy(self.motorcontroller.pos)
                startpos.theta = (relative_position.getArgRad() - math.pi/2) + self.motorcontroller.pos.theta
                self.motorcontroller.setStraightPID(speed_abs=60,dest=abs(relative_position),
                                                    startpos=startpos,
                                                    kparam_d=0.0,
                                                    kparam_p=0.01,
                                                    kparam_i=0.005,
                                                    complete_delta=30)
                while not self.motorcontroller.isComplete() and self.mainthread.isAlive():
                    self.motorcontroller.run()
                    
                    #壁検知フラグをセット、検知しなければループ続行。
                    self.wall_flag = self.checkWall()
                    if True in self.wall_flag.values(): break
                self.motorcontroller.brake()
                
            
            #怪盗を捕まえたら、終了モードに遷移する
            #if
            #    self.image.stop()
            #    self.mode = "finish"
            #    return

    def route_compensate(self):
        self.fpgaio.put((1,0,0,0))
        print("route_compensate start")
        if len(self.search_route) == 0:
            self.nextpos = self.oldpos
        else:
            self.nextpos  = self.search_route.searchNearAndSetIndex(self.motorcontroller.pos)
        while self.mainthread.isAlive():
            print "route compensate to "+str(self.nextpos)
            self.oldpos = self.motorcontroller.pos
            self.motorcontroller.setStraight(speed_abs=5, dest=-100, kparam_d=0.0, complete_delta=30)
            while not self.motorcontroller.isComplete() and self.mainthread.isAlive():
                self.motorcontroller.run()
            self.motorcontroller.brake()
            self.rotateToPosition(  endpos=self.nextpos, startpos=self.motorcontroller.pos) #目的位置を向くように回転
            self.straightToPosition(endpos=self.nextpos, startpos=self.motorcontroller.pos) #障害物を検知したら中断する

            #壁検知フラグが立っていたら、もとの位置まで戻る。
            if True in self.wall_flag.values():
                self.position_compensate()
            else:
                break
        print("route_compensate end")
        print self.search_route
        self.mode = "search"
        
    def run(self):
        while self.fpgaio.get(hardwareconst.TOUCH["c"]) == 0: pass
        while self.mainthread.isAlive():
            if   self.mode == "patrol":     self.patrol()
            elif self.mode == "search":     self.search()
            elif self.mode == "chase":      self.chase()
            elif self.mode == "route_compensate": self.route_compensate()
            elif self.mode == "finish":     break
            else:                           raise ValueError
        print("finish!")
