#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
ImageThreadを作成・操作するモジュール
"""
import position, math
import imagethread, camera
import virtualimagethread
import os.path

#debugをTrueにすると、画像出力あり
class Image:
    def __init__(self, jpegfile_directory="/dev/shm/", debug=True, mode="real"):
        option = "-m"
        """#5 camera
        camera_positions  = [position.Position(0,100,90.0*math.pi/180),position.Position(100,100,60.0*math.pi/180),position.Position(-100,100,135.0*math.pi/180),position.Position(100,0,0.0*math.pi/180),position.Position(-100,0,180.0*math.pi/180)]
        cameras = [camera.Camera(camera_positions[i],
                                 index=i,
                                 debug=debug) for i in range(5)]
        """
        
        #three camera
        camera_positions  = [position.Position(100,0,0.0*math.pi/180),position.Position(0,100,90.0*math.pi/180),position.Position(-100,0,180.0*math.pi/180)]
        cameras = [camera.Camera(position=camera_positions[i],
                                 index=i,
                                 debug=debug) for i in range(3)]
        
        """#two camera
        camera_positions  = [position.Position(0,100,90.0*math.pi/180),position.Position(100,0,0.0*math.pi/180)]
        sizes = [(480,640),(480,640)]
        cameras = [camera.Camera(position=camera_positions[i],
                                 index=i,
                                 debug=debug) for i in range(2)]
        
        #single camera for debug
        """
        """cameras = [camera.Camera(position.Position(0,100,90.0*math.pi/180),
                                 index=0,
                                 debug=debug,
                                 size=(120,160))]
        """
        
        if mode == "virtual": self.imagethread = virtualimagethread.VImageThread(cameras, debug)
        elif mode == "real":  self.imagethread = imagethread.ImageThread(cameras,debug)
        else: raise ValueError

    def precapture(self):
        for camera in self.imagethread.cameras:
            camera.precapture()
    def start(self, mode="all"):
        self.imagethread.thread_mode = mode
    def stop(self):
        self.imagethread.thread_mode = "stop"
    def getNumbers(self):
        return self.imagethread.getNumbers()
    def isBall(self):
        return self.imagethread.isBall()
    def getBallRelativePosition(self):
        return self.imagethread.getBallRelativePosition()
    def snapshot(self):
        self.imagethread.snapshot()

if __name__=="__main__":
    import time
    img = Image(debug=False, mode="real")
    img.precapture()
    img.start(mode="ball")
    while raw_input("Input quit (to finish) or Enter(to get values)...") != "quit":
        print img.getBallRelativePosition()
    img.stop()
    """
    import timeit
    number = 1
    print("getimage : "+str(timeit.timeit("[img.imagethread.cameras[i]._loadImageFile() for i in range(3)]","from __main__ import img", number=number)/number) + "[s]")
    print("ball   : " + str(timeit.timeit("img.imagethread._update(\"ball\")","from __main__ import img", number=number)/number) + "[s]")
    print("number : " + str(timeit.timeit("img.imagethread._update(\"number\")","from __main__ import img", number=number)/number) + "[s]")
    """
