#!/usr/bin/env python
# -*- coding: utf-8 -*-
import threading,position

class ImageThread(threading.Thread):
    def __init__(self, cameras, debug=True):
        threading.Thread.__init__(self)
        self.cameras = cameras
        self.thread_mode = "stop" #ball,number,stopの3種類
        self.mainthread = threading.currentThread()
        threading.Thread.start(self)
    def run(self):
        while self.mainthread.isAlive():
            self._update(self.thread_mode)
    def _update(self,mode):
        if mode == "ball":   self._update_ball()
        elif mode == "number": self._update_number()
        elif mode == "stop":
            for camera in self.cameras:
                camera.clear()
        else : pass
    def _update_ball(self):
        for camera in self.cameras:
            camera.update_ball()
    def _update_number(self):
        for camera in self.cameras:
            camera.update_number()

    def getNumbers(self,count=2):
        for camera in self.cameras:
            if len(camera.getNumbers()) == count:
                return camera.getNumbers()
        return []
    def isBall(self):
        return any([camera.isBall() for camera in self.cameras])
    def getBallRelativePosition(self):
        positions = [camera.getBallRelativePosition() for camera in self.cameras if camera.isBall()]
        if positions == []: return position.Position(0,0)
        distances = [abs(positions[i]) for i in range(len(positions))]
        minimam_distance_index = distances.index(min(distances))
        return positions[minimam_distance_index]
    def snapshot(self):
        for camera in self.cameras:
            camera.snapshot()
