#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
このモジュールはMIRS1302におけるトップクラスの処理を定義している。
"""
import sys, time
import mirsgui
import ussthread, image, controllerthread
import motorcontroller, fpgaio, field, route, hardwareconst, datasender
import virtualfpgaio

GUI_FLAG = True
#GUI_FLAG = False
def main():
    """メイン関数"""
    
    aussthread = ussthread.UssThread(serialport="/dev/ttyS0", 
                                     mode="real", uss_count=4)
    amotorcontroller = motorcontroller.MotorController(
        path_l=hardwareconst.MOTOR_L,
        path_r=hardwareconst.MOTOR_R, mode="real")
    aimage = image.Image(debug=False, mode="real")
    afpgaio = fpgaio.FpgaIO("/dev/fpga_io")
    
    """
    aussthread = ussthread.UssThread(serialport="/dev/ttyS0", 
                                     mode="virtual", uss_count=4)
    amotorcontroller = motorcontroller.MotorController(
        path_l=hardwareconst.MOTOR_L,
        path_r=hardwareconst.MOTOR_R, mode="virtual")
    aimage = image.Image(debug=False, mode="real")
    afpgaio = virtualfpgaio.VirtualFpgaIO("/dev/fpga_io")
    """
    afield = field.Field("data/field.dat")
    patrol_route = route.Route("data/patrol_route.dat")
    search_route = route.Route("data/search_route.dat")
    adatasender = datasender.DataSender()
    #adatasender = None
    acontrollerthread = controllerthread.ControllerThread(
        aussthread, amotorcontroller, aimage,
        afpgaio, afield, patrol_route, search_route, adatasender)

    if GUI_FLAG:
        amirsgui = mirsgui.MirsGUI(acontrollerthread, "real")

    aussthread.start()
    acontrollerthread.start()

    #待機する。これを呼び出さないと、他のスレッドは全て終了してしまう。GUIを使用する場合は、この間にGUIの処理を行う。
    if GUI_FLAG:
        sys.exit(amirsgui.exec_())
    else:
        while True:
            time.sleep(1000)

if __name__ == "__main__":
    main()
