#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""このモジュールはGUI処理を定義している。"""
import math, sys
from PyQt4 import QtGui
from PyQt4 import QtCore
import position
import hardwareconst

SCALE = 10
CENTER_X = 250
CENTER_Y = 500
class MirsGUI(QtGui.QWidget):
    """
    GUIによるMIRSの状態表示を行うためのクラス
    MIRSの内部状態を知るために、controllerthreadを包含する。
    """
    def __init__(self, controllerthread, mode="real"):
        self.app = QtGui.QApplication(sys.argv)
        QtGui.QWidget.__init__(self)
        self.mode = mode
        self.controllerthread = controllerthread
        self.resize(1000, 600)
        self.setWindowTitle("mirs on GUI")
        if self.mode == "virtual":
            self.button1 = QtGui.QPushButton("ball find", self)
            self.button1.clicked.connect(self._toggleIsBallFlag)
            self.button1.setGeometry(800,0,100,20)
            self.button2 = QtGui.QPushButton("left touch", self)
            self.button2.clicked.connect(self._toggleLeftTouch)
            self.button2.setGeometry(800,20,100,20)
        self.labels = [QtGui.QLabel(self) for i in range(10)]
        for i in range(10):
            self.labels[i].setGeometry(500,20*i,300,20)
        self.timer = QtCore.QTimer()
        self.timer.connect(self.timer, QtCore.SIGNAL("timeout()"), self.update)
        self.timer.start(100)

    def update(self):
        if self.mode == "virtual":
            self.labels[1].setText("isBall : " + str(self.controllerthread.image.imagethread.isball))
        self.labels[0].setText("mode : " + self.mode)
        self.labels[2].setText("position : " + str(self.controllerthread.motorcontroller))
        self.labels[3].setText("wall : " + str(self.controllerthread.wall_flag))
        self.labels[4].setText("uss : (" + str(hardwareconst.USS_POS) + ")" + str(self.controllerthread.ussthread))
        QtGui.QWidget.update(self)
    def exec_(self):
        QtGui.QWidget.show(self)
        return self.app.exec_()
    def paintEvent(self, dummy_e):
        """描画イベントの処理"""
        self.painter = QtGui.QPainter()
        self.painter.begin(self)
        brush = QtGui.QBrush(QtCore.Qt.SolidPattern)
        self.painter.setBrush(brush)
        self.drawCourse()
        self.drawMachine()
        if self.mode == "virtual":
            self.calcVirtualPhantom()
            self.drawPhantom()
        self.painter.end()
    def calcVirtualPhantom(self):
        """怪盗の位置を偽装する"""
        self.controllerthread.image.imagethread.calcVirtualPhantom(self.controllerthread.motorcontroller)
    def drawCourse(self):
        self.painter.drawLine(CENTER_X, 0,
                              CENTER_X, 500)
        self.painter.drawLine(0, CENTER_Y,
                              500, CENTER_Y)
        for i in range(-4,6):
            for j in range(10):
                self.painter.drawPoint(CENTER_X + 500*i/SCALE,
                                       CENTER_Y - 500*j/SCALE)
    def drawMachine(self):
        """マシンを描画する"""
        pos = self.controllerthread.motorcontroller.pos
        theta = pos.getThetaRad()
        self.painter.drawRect(CENTER_X + pos.x/SCALE,
                              CENTER_Y - pos.y/SCALE, 2, 2)
        self.painter.drawLine(CENTER_X + pos.x/SCALE,
                              CENTER_Y - pos.y/SCALE,
                              CENTER_X + pos.x/SCALE + 10.0*math.cos(theta),
                              CENTER_Y - pos.y/SCALE - 10.0*math.sin(theta))
    def drawPhantom(self):
        """怪盗を描画する"""
        pos = self.controllerthread.image.imagethread.phantompos
        theta = pos.getThetaRad()
        self.painter.drawRect(pos.x/SCALE, pos.y/SCALE, 2, 2)

    def _toggleIsBallFlag(self):
        """MIRSがボールを発見しているかどうかを誤魔化して、見つけたと思わせる。"""
        self.controllerthread.image.imagethread.toggleIsBallFlag()
    def _toggleLeftTouch(self):
        self.controllerthread.fpgaio.toggle(hardwareconst.TOUCH["l"])
