#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""位置情報を扱うためのモジュール"""
import math
def periodicRestrict(target_value, from_value, to_value):
    """target_valueをfrom_valueからto_valueの間に制限したものを返す。角度などの制限に使用する。"""
    while target_value >= to_value:
        target_value -= to_value - from_value
    while target_value <= from_value:
        target_value += to_value - from_value
    return target_value

class Position(object):
    """位置情報を扱うためのクラス。x座標,y座標と角度thetaを持つ。角度thetaはMIRSの向きを表す"""
    def __init__(self, x, y, theta=0):
        self.x = x
        self.y = y
        self.theta = theta
    def __abs__(self):
        return math.sqrt(self.x**2+self.y**2)
    def __str__(self):
        return ("(" + str.format("{0:.2f}", self.x) +
                 ", " + str.format("{0:.2f}", self.y) +
                 ", " + str.format("{0:.2f}", self.theta/math.pi*180) + ")")
    def __sub__(self, position):
        return Position(self.x - position.x, self.y - position.y)
    def __sum__(self, position):
        return Position(self.x + position.x, self.y + position.y)
    def getThetaDeg(self):
        """角度値を度数法で返す"""
        return self.theta/math.pi*180
    def getThetaRad(self):
        """角度値を弧度法で返す"""
        return self.theta
    def getArgDeg(self):
        """偏角を度数法で返す。戻り値は -180 から 180 の間"""
        return math.atan2(self.y, self.x)/math.pi*180
    def getArgRad(self):
        """偏角を弧度法で返す。戻り値は -pi から pi の間"""
        return math.atan2(self.y, self.x)
