#!/usr/bin/env python
# -*- coding: utf-8 -*-
import threading,logging,time,os,select,termios

NEWTIO_CC = [0 for i in range(termios.NCCS)]
NEWTIO_CC[termios.VMIN] = 1
NEWTIO_CFLAG = termios.B9600 | termios.CS8 | termios.CLOCAL | termios.CREAD
NEWTIO_IFLAG = termios.IGNPAR

class UssThread(threading.Thread):
	def __init__(self,fd_path,quantity):
		threading.Thread.__init__(self)
		self.run_flag = True
		self.quantity = quantity
		self.value = [0 for i in range(quantity)]
		self.fd=os.open(fd_path, os.O_RDWR | os.O_NOCTTY | os.O_NONBLOCK)
		self.oldtio=None

	def __del__(self):
		os.close(self.fd)

	def run(self):
		if self.fd < 0:
			sys.stderr.write("Uss fd error")
			return
		while self.run_flag == True:
			for i in range(self.quantity):
				self.value[i] = self._read(i)
				time.sleep(0.038) #隣り合うUSSが干渉するため30msec以上waitを入れる。

	def finish(self):
		self.run_flag = False
		self.join()

	def _read(self,uss_id):
		if uss_id < 0 or uss_id >= self.quantity :
			sys.stderr.write("Uss _read error: uss_id is bad.\n")
			return -1

		self._setup_term()
		os.write(self.fd, chr(uss_id+1))
		buf0 = self._get_key()
		buf1 = self._get_key()
		if buf1 == 0xff:
			sys.stderr.write("uss read: 0xff.\n")
			return -1
		self._cleanup_term()

		return (ord(buf1)<<8) + ord(buf0) + 20

	def _get_key(self):
		res = select.select([self.fd],[],[],0.05)
		if res == ([],[],[]) :
			return 0
		return os.read(self.fd,1)

	def _setup_term(self):
		self.oldtio = termios.tcgetattr(self.fd)
		termios.tcflush(self.fd, termios.TCIFLUSH)
		termios.tcsetattr(self.fd, termios.TCSANOW, [NEWTIO_IFLAG,0,NEWTIO_CFLAG,0,0,0,NEWTIO_CC])

	def _cleanup_term(self):
		if self.oldtio is not None:
			termios.tcsetattr(self.fd, termios.TCSANOW, self.oldtio)

if __name__ == "__main__":
	FD_PATH = "/dev/ttyS0"
	QUANTITY = 2
	ussthread = UssThread(FD_PATH,QUANTITY)
	ussthread.setDaemon(True)
	ussthread.start()

	time.sleep(1)
	for i in range(5):
		for j in range(QUANTITY):
			print(ussthread.value[j])
		time.sleep(1)

	ussthread.finish()

