#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest,time,random,copy
import math
import ussthread,virtualmotor,motorcontroller,balldetector,numberdetector,image,fpgaio,field,route,hardwareconst

class TestCaseAll(unittest.TestCase):
    def setUp(self):
        pass
    def tearDown(self):
        pass
        
    def test_uss(self):
        with self.assertRaises(ValueError):
            ussthread.UssThread(serialport="/dev/ttyS0",uss_count=-1,mode="virtual")
        with self.assertRaises(ValueError):
            ussthread.UssThread(serialport="/dev/ttyS0",uss_count=0,mode="virtual")
            
        aussthread = ussthread.UssThread(serialport="/dev/ttyS0",uss_count=2,mode="virtual")
        aussthread.start()

        #値が変化することのテスト(失敗したらハードウェアの問題の可能性がある．)
        for i in range(100):
            old_values = copy.deepcopy(aussthread.values)
            time.sleep(0.01)
            if aussthread.values != old_values : break
        else: #if not break
            self.fail("value is stack")

        #値がある範囲に収まっていることのテスト
        for i in range(100):
            self.assertFalse(0 in aussthread.values)
            for value in aussthread.values: self.assertTrue(20 <= value <= 400 or value==9999)
            time.sleep(0.01)

        aussthread.stop()
        
    def test_motor(self):
        amotor = virtualmotor.VirtualMotor("/dev/mirS0")
        
        amotor.clear()
        dest=10
        complete_delta=5
        while not amotor.isComplete(dest,complete_delta):
            amotor.control(speed=1)
            amotor.update()
        assert dest-complete_delta <= amotor.distance <= dest+complete_delta
        amotor.clear()
        dest=-10
        complete_delta=5
        while not amotor.isComplete(dest,complete_delta):
            amotor.control(speed=-1)
            amotor.update()
        assert dest-complete_delta <= amotor.distance <= dest+complete_delta
        
        with self.assertRaises(AssertionError):
            amotor.isComplete(dest=0,complete_delta=-1)
        
    def test_motorcontroller(self):
        amotorcontroller = motorcontroller.MotorController(path_l = "/dev/mirS0", path_r = "/dev/mirS1", mode="virtual")
        nextpos=[0,100]
        dx = nextpos[0]-amotorcontroller.pos.x
        dy = nextpos[1]-amotorcontroller.pos.y
        amotorcontroller.clear()
        while not amotorcontroller.straight(speed=10,dest=math.sqrt(dx**2+dy**2),kd=0.01,kp=1,ki=1.0,complete_delta=12):
            pass #print(amotorcontroller.pos.x,amotorcontroller.pos.y,amotorcontroller.pos.theta/math.pi*180)
        
    def test_imagethread(self):
        aimagethread     = image.ImageThread("data/input.jpg")
        
#    def test_fpgaio(self):
#        afpgaio          = fpgaio.FpgaIO("/dev/fpga_io")
        
    def test_field(self):
        afield = field.Field("data/field.dat")
        
    def test_patrol_route(self):
        apatrol_route = route.Route("data/patrol_route.dat")
        
    def test_search_route(self):
        asearch_route = route.Route("data/search_route.dat")

    
if __name__ == "__main__":
    unittest.main(verbosity=2)