#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""超音波距離センサを扱うためのモジュール"""
import serial

class Uss(object):
    """超音波距離センサを扱うためのクラス"""
    def __init__(self, serialport, identity_number):
        self.serialio = self._open_serialport(serialport)
        self.id = identity_number
        self.old_value = 20
    def __del__(self):
        self.serialio.close()
    def _open_serialport(self, serialport):
        return serial.Serial(port=serialport, parity=serial.PARITY_NONE,
                              timeout=0.1)

    def read(self):
        """超音波距離センサに計測を命令し、返ってきた距離情報を読み込む"""
        self.serialio.write(chr(self.id+1))
        buf0 = self.serialio.read(1)
        buf1 = self.serialio.read(1)
        if buf0 == "" or buf1 == "":
            return self.old_value
        value = (ord(buf1)<<8) + ord(buf0) + 20
        self.old_value = value
        return value

def simple_hw_test():
    """簡単なハードウェアテスト"""
    import time
    uss = [Uss("/dev/ttyS0", identity_number=i) for i in range(4)]
    num = input("num=? >")
    while True:
        print str(num) + ":" + str(uss[num].read())
        time.sleep(0.1) #ゆっくりにする

if __name__ == "__main__":
    simple_hw_test()
