#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""超音波距離センサによる計測を行うスレッドを定義するモジュール"""
import threading, time, virtualuss, uss

class UssThread(threading.Thread):
    """超音波距離センサによる計測を行うスレッドクラス"""
    def __init__(self, serialport, uss_count, wait=0.0, mode="real"):
        threading.Thread.__init__(self)
        if uss_count <= 0:
            raise ValueError
        if uss_count != 4:
            print "warning: uss_count != 4"
        if mode == "virtual":
            self.uss = [virtualuss.VirtualUss(serialport, identity_number=i)
                        for i in range(uss_count)]
        elif mode == "real":
            self.uss = [uss.Uss(serialport, identity_number=i)
                        for i in range(uss_count)]
        else:
            raise ValueError
        self.uss_count = uss_count
        self.values = [0 for i in range(uss_count)]
        self.stop_flag = False
        self.wait = wait
        self.mainthread = threading.currentThread()

    def run(self):
        while self.mainthread.isAlive() and not self.stop_flag:
            for i in range(len(self.uss)):
                self.values[i] = self.uss[i].read() #約30msかかる
                time.sleep(self.wait) #節電?効果は未知

    def __str__(self):
        return str(self.values)
    def start(self):
        assert self.mainthread == threading.currentThread(), "permission denied"
        threading.Thread.start(self)
        time.sleep(self.wait*self.uss_count) #最初の計測が終わるまで待つ
    def stop(self):
        """スレッドを停止する"""
        self.stop_flag = True
        time.sleep(self.wait*self.uss_count) #計測が終わるまで待つ
