#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Motorクラスのスタブを定義するモジュール"""
import struct, time, threading, motor

class VirtualMotor(motor.Motor):
    """Motorクラスのスタブ"""
    DELTA_TIME = 0.1
    SIMULATION_SPEED = 5
    mainthread = threading.currentThread()
    def __init__(self, path):
        self.real_speed = 0 #[cm]で扱う
        self.time = 0
        self.total_puls = 0 #[度]で扱う
        self.desire_speed = 0 #[mm]で扱う
        self.vkp = self.MCB_KP
        self.vki = self.MCB_KI
        motor.Motor.__init__(self, path)
        self.thread = threading.Thread(target=self._virtual_mcb_update)
        self.thread.start()

    def _virtual_mcb_update(self):
        """スタブのための更新を行う"""
        while self.mainthread.isAlive():
            self.real_speed = self.desire_speed
            puls_per_time = self.real_speed / self.distance_per_single_puls
            self.total_puls += puls_per_time * self.DELTA_TIME
            self.time += self.DELTA_TIME
            time.sleep(self.DELTA_TIME/self.SIMULATION_SPEED)
    def _open_path(self, flag):
        return 4 #なんでもいいから値を返す
    def _close(self):
        pass
    def _start(self):
        pass
    def _stop(self):
        self.control(0)
    def _request(self, send_data, response_size):
        assert response_size > 0, "response_size is wrong"
        if send_data[0] == chr(self.PKT_SETCONTROL):
            #struct.unpackで必ずデータのロスが発生する。
            #スピード、Kパラメータは整数にされる
            send = struct.unpack("BbBB", send_data)
            self.desire_speed = 10*send[1]
            self.vkp = send[2]
            self.vki = send[3]
            time.sleep(self.DELTA_TIME/10/self.SIMULATION_SPEED)
        elif send_data == chr(self.PKT_GETSTATUS):
            return struct.pack("BbHi", 0, 0, 0, self.total_puls)
        elif send_data == chr(self.PKT_CLEAR):
            self.total_puls = 0
        return [dummydata for dummydata in range(response_size)]
